/******************************************************************************
 *
 * Filename: char.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  4th March 2002
 *
 * Purpose:  Include file for char instructions.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

int caload( Stack* stp )
{
	Frame *fp = stp->fp;
	int32 index;
	ObjRef* array;
	u1* data;

	INSTRUCTION( "%p: caload\n", fp->pc );

	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	if( !array )
		return newCompleteException( stp, NullPointerException, "caload" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || ((array->type != T_CHAR) && (array->type != T_BYTE)) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad character array" );

	data = (u1*) array->data;

	PUSH_INT( fp, data[index] );

	return 0;
}


int castore( Stack* stp )
{
	Frame *fp = stp->fp;
	u4 value;
	int32 index;
	ObjRef* array;
	u1* data;

	INSTRUCTION( "%p: castore\n", fp->pc );

	POP_INT( fp, value );
	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	if( !array )
		return newCompleteException( stp, NullPointerException, "castore" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || ((array->type != T_CHAR) && (array->type != T_BYTE)) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad character array" );

	data = (u1*) array->data;

	data[index] = (u1) value;

	return 0;
}
