/******************************************************************************
 *
 * Filename: verifier.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Class verifier for passes 2 & 3.
 *
 * (c) Copyright R.H.Thornburrow, 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "config.h"

#include "verifier.h"
#include "disasm.h" 


int verifyPass2( ClassFile* cf, Stack* stp )
{
	/* All classes except object must have a superclass */
	if( !cf->super && !(cf->access_flags & ACC_INTERFACE) )
	{
		if( strcmp( cf->name, "java/lang/Object" ) != 0 )
		{
			printf( "VERIFY ERROR: Missing superclass for class \"%s\"\n", cf->name );
			stp->retcode = newCompleteException( stp, VerifyError, "Missing superclass" );
			return 0;
		}
	}

	/* Superclass must not be final */
	if( cf->super )
	{
		if( cf->super->access_flags & ACC_FINAL )
		{
			printf( "VERIFY ERROR: Attempt to override final class in class \"%s\"\n", cf->name );
			stp->retcode = newCompleteException( stp, VerifyError, "Attempt to override final class" );
			return 0;
		}
	}

	return 1;
}
